@@ -17,6 +17,10 @@
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 
+#define RET_ERROR -1
+#define RET_WARNING 1
+#define RET_SUCCESS 0
+
 using namespace llvm;
 
 std::random_device rd;
@@ -571,7 +575,7 @@ int replaceOperands(StringRef dst_desc, StringRef src_desc, Module &M)
         if (DV->getType() != SV->getType())
             return -4;
         if (DV == SV)
-            return -5;
+            return 1;
     }
     else // Constant value
         SV = getConstantValue(DV->getType());
