@@ -5,13 +5,41 @@ import subprocess
 import inspect
 import os
 
-import gevo
-from gevo._llvm import __llvm_version__
+try:
+    import gevo._llvm
+    from gevo import __version__
+    from gevo._llvm import __llvm_version__
+    from gevo._llvm import __llvm_bin_folder__
 
-os.path.dirname(inspect.getfile(gevo))
-LLVM_MUTATE_LIBRARY_PATH=f'{os.path.dirname(inspect.getfile(gevo))}/Mutate.so.{__llvm_version__}'
+except ImportError:
+    # TODO: load mutation.so from Mutate folder with proper llvm version string as a standalone mode
+    print('Cannot find Mutate.so because gevo is not installed! ')
+    sys.exit(-1)
+
+class FileOpener(argparse.FileType):
+    # delayed FileType;
+    # sample use:
+    # with args.input.open() as f: f.read()
+    def __call__(self, string):
+        # optionally test string
+        self.filename = string
+        return self
+    def open(self):
+        return super(FileOpener,self).__call__(self.filename)
+    file =  property(open, None, None, 'open file property')
 
 if __name__ == '__main__':
+    LLVM_MUTATE_LIBRARY_PATH = f'{os.path.dirname(inspect.getfile(gevo._llvm))}/Mutate.so.{__llvm_version__}'
+    opt_path = f'{__llvm_bin_folder__}/opt'
+    llvm_dis_path = f'{__llvm_bin_folder__}/llvm-dis'
+
+    if os.path.isfile(opt_path) is False:
+        print(f'{opt_path} does not exist!')
+        sys.exit(-1)
+    if os.path.isfile(llvm_dis_path) is False:
+        print(f'{llvm_dis_path} does not exist!')
+        sys.exit(-1)
+    
     # Command parser
     class MutationAction(argparse.Action):
         def __call__(self, parser, namespace, values, option_string=None):
@@ -22,43 +50,48 @@ if __name__ == '__main__':
             setattr(namespace, 'mutation_commands', previous_act)
 
     llvm_mutate_parser = argparse.ArgumentParser(description="Manipulate LLVM assembly from Nvidia CUDA Kernels")
-    llvm_mutate_parser.add_argument('-o', '--output_file', metavar='FILE', nargs='?', type=argparse.FileType('w'), default=sys.stdout,
-        help='Output file name. Output will be redirected to stdout if output file name is not specified')
-    llvm_mutate_parser.add_argument('-f', '--input_file', metavar='FILE', nargs='?', type=argparse.FileType('rb'), default=sys.stdin,
-        help='Input file name. llvm-mutate will use and wait for stdin as the input stream if input file name is not specified.')
+    llvm_mutate_parser.add_argument('-o', '--output_file', metavar='FILE', type=FileOpener('w'), default='-',
+        help='output file name. Output will be redirected to stdout if output file is not specified')
+    llvm_mutate_parser.add_argument('-f', '--input_file', metavar='FILE', type=FileOpener('r'), default='-',
+        help='input file name. llvm-mutate will use and wait for stdin as the input stream if input file is not specified.')
     llvm_mutate_parser.add_argument('-n', '--name', action='store_true',
-        help='Add unique ID (UID) for each instruction. UID is one of needed instruction description in mutation operations')
+        help='add unique ID (UID) for each instruction. UID is one of needed instruction description in mutation operations')
     llvm_mutate_parser.add_argument('-I', '--ids', action='store_true',
-        help='Show the number of instructions')
+        help='show the number of instructions')
+    llvm_mutate_parser.add_argument('-q', '--query', type=str, metavar='INST',
+        help='query the instruction\'s source line info')
     llvm_mutate_parser.add_argument('--not_use_result', action='store_true',
-        help='Not connect the newly inserted instruction\'s result value back into the use-def\
+        help='not connect the newly inserted instruction\'s result value back into the use-def\
               chain when performing mutation operations. This argument is mainly for reproducing\
               program variant from a sequence of mutations')
+    llvm_mutate_parser.add_argument('-V', '--version', action='version', version='%(prog)s under gevo-' + __version__)
 
     # grouping mutation commands
     mutation_operation_group = llvm_mutate_parser.add_argument_group(
         title='Mutation Operations',
         description='Mutation operations only accept instruction description [INST] in 2 formats: \
-                     integer number as instruction index or Unique ID. \
-                     Note: --name, --ids, and mutation operation cannot be used together')
+                     integer number as instruction index or Unique ID. The advantage of UID is,  \
+                     once created, persistence throughout the mutation operation as long as the \
+                     the instruction UID denoted is not the target of mutation operation. \
+                     Note: --name, --ids, and mutation operation cannot be used together.')
     mutation_operation_group.add_argument(
         '-c', '--cut', type=str, dest='-cut', action=MutationAction, metavar='INST',
-        help='Cut the given instruction')
+        help='cut the given instruction')
     mutation_operation_group.add_argument(
         '-i', '--insert', type=str, dest='-insert', action=MutationAction, metavar='INST1,INST2',
-        help='Copy the second inst. before the first')
+        help='copy the second inst. before the first')
     mutation_operation_group.add_argument(
         '-p', '--oprepl', type=str, dest='-oprepl', action=MutationAction, metavar='INST1,INST2',
-        help='Replace the first Operand. with the second')
+        help='replace the first Operand. with the second')
     mutation_operation_group.add_argument(
         '-r', '--replace', type=str, dest='-replace', action=MutationAction, metavar='INST1,INST2', 
-        help='Replace the first inst. with the second')
+        help='replace the first inst. with the second')
     mutation_operation_group.add_argument(
         '-m', '--move', type=str, dest='-move', action=MutationAction, metavar='INST1,INST2',
-        help='Move the second inst. before the first')
+        help='move the second inst. before the first')
     mutation_operation_group.add_argument(
         '-s', '--swap', type=str, dest='-swap', action=MutationAction, metavar='INST1,INST2',
-        help='Swap the location of two instructions')
+        help='swap the location of two instructions')
 
     args = llvm_mutate_parser.parse_args()
     OPT_NOT_USE_RESULT = '-use_result=0' if args.not_use_result else ''
@@ -69,10 +102,10 @@ if __name__ == '__main__':
 
     if args.name:
         opt_proc = subprocess.Popen(
-            [ f'opt-{__llvm_version__}',
+            [ opt_path,
               '-load', LLVM_MUTATE_LIBRARY_PATH,
               '-name'],
-            stdin=args.input_file,
+            stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
         opt_stdout, opt_stderr = opt_proc.communicate()
@@ -82,10 +115,10 @@ if __name__ == '__main__':
             sys.exit(-1)
     elif args.ids:
         opt_proc = subprocess.Popen(
-            [ f'opt-{__llvm_version__}',
+            [ opt_path,
               '-load', LLVM_MUTATE_LIBRARY_PATH,
               '-ids'],
-            stdin=args.input_file,
+            stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
         opt_stdout, opt_stderr = opt_proc.communicate()
@@ -95,16 +128,32 @@ if __name__ == '__main__':
             sys.exit(-1)
         print(opt_stderr.decode(), end='', file=sys.stderr)
         sys.exit(0)
+    elif args.query:
+        opt_proc = subprocess.Popen(
+            [ opt_path,
+              '-load', LLVM_MUTATE_LIBRARY_PATH,
+              '-query', f'-inst1={args.query}'],
+            stdin=args.input_file.open(),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE )
+        opt_stdout, opt_stderr = opt_proc.communicate()
+        if opt_proc.returncode != 0:
+            print(f"llvm-mutate: Error in {opt_proc.args}")
+            print(opt_stderr.decode(), end='')
+            sys.exit(-1)
+        print(opt_stderr.decode(), end='', file=sys.stdout)
+        sys.exit(0)
     elif 'mutation_commands' in args:
-        input_str = args.input_file.buffer.read()
+        with args.input_file.open() as f:
+            input_str = f.buffer.read()
         for mop in args.mutation_commands:
             insts = mop[1].split(',')
             inst_args = [ f'-inst1={insts[0]}' ] if len(insts) == 1 else\
                         [ f'-inst1={insts[0]}', f'-inst2={insts[1]}']
-            opt_args = [f'opt-{__llvm_version__}',
+            opt_args = [opt_path,
                         '-load', LLVM_MUTATE_LIBRARY_PATH,
                         '-use_result=0', mop[0]] if args.not_use_result else\
-                       [f'opt-{__llvm_version__}',
+                       [opt_path,
                         '-load', LLVM_MUTATE_LIBRARY_PATH, mop[0]]
 
             opt_args.extend(inst_args)
@@ -135,7 +184,7 @@ if __name__ == '__main__':
         sys.exit(-1)
 
     llvmdis_proc = subprocess.Popen(
-        [ f'llvm-dis-{__llvm_version__}'],
+        [ llvm_dis_path ],
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE )
@@ -145,7 +194,7 @@ if __name__ == '__main__':
         print(llvmdis_stderr.decode())
         sys.exit(-1)
 
-    print(llvmdis_stdout.decode(), file=args.output_file, end='')
+    print(llvmdis_stdout.decode(), file=args.output_file.open(), end='')
 
     # Link to PTX
     # cuda.init()
